package Wifidelity::80211;

use 5.008008;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

our @EXPORT_OK = ('data_src', 'data_bssid');

our @EXPORT = qw(%c80211types %c80211subtypes %c80211ds %c80211bin
	
);

our $VERSION = '1';

# 802.11 constants
our %c80211types = ( 
  MAN => 0x0,
  DATA => 0x2,
);

our %c80211subtypes = ( 
  BEACON => 0x8,
  NULL => 0x4,
  QOS => 0x8
);

our %c80211ds = ( 
  TO => 0x1,
  FROM => 0x2
);

our %c80211bin = (
  FRAME_CONTROL => 0,
  SEQNUM => 22,
  SEQNUM_SHIFT => 4,
  TIMESTAMP => 24,
  A1 => 4,
  A2 => 10,
  A3 => 16,
  A4 => 22,
  HEADER_ID => 4,
  AVS_HEADER_ID => 0x40,
  PRISM_HEADER_ID => 0x90000000,
  AVS_HEADER_LENGTH => 0x40,
  PRISM_HEADER_LENGTH => 0x90,
  TYPE_SHIFT => 2,
  TYPE_MASK => 0x3,
  DS_MASK => 0x03,
  SUBTYPE_SHIFT => 4,
  SUBTYPE_MASK => 0xf,
  RETRY_SHIFT => 3,
  RETRY_MASK => 0x8,
);

# 802.11 data structures
#
# %80211_packet = ( 
#   TYPE,
#   SUBTYPE,
#   BSSID_MAC,
#   SRC_MAC, #   DST_MAC,
#   SEQNUM,
#   RETRY,
#   TIME,
#   BEACON_TIME
# ) 

sub data_src {
  my ($a1, $a2, $a3, $ds, $bmcast_mac) = @_;
  my ($dst);
  if ($ds == $c80211ds{TO}) {
    return $a2; 
  }
  if ($ds == $c80211ds{FROM}) {
    $dst = $a1;
    # Use a2 if broadcast or multicast
    if ($bmcast_mac && !($dst =~ m/^00:/)) {
      return $a2;
    }
    return $a3
  }
  return $a2
}

sub data_bssid {
  my ($a1, $a2, $a3, $ds) = @_;
  if ($ds == $c80211ds{TO}) {
    return $a1;
  }
  if ($ds == $c80211ds{FROM}) {
    return $a2;
  }
 return $a3;
}

1;
__END__

=head1 NAME

Wifidelity::Stats - Perl extension for blah blah blah

=head1 SYNOPSIS

  use Wifidelity::80211;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Wifidelity::80211, created by h2xs. 

=head2 EXPORT

=head1 SEE ALSO

=head1 AUTHOR

Aaron Schulman, E<lt>schulman@cs.umd.edu<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Aaron Schulman

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.


=cut
