package Wifidelity::Pcap;

use Wifidelity::80211;

use 5.008008;
use strict;
use warnings;

use Net::Pcap;
use Wifidelity::80211 qw(data_src data_bssid);
use Config;

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT_OK = qw(pcap_trace pcap_live is_64_bit);

our @EXPORT = qw();

our $VERSION = '0.01';

# $packet_config = ( 
#   packet_callback,
#   callback_data,
#   callback_config,
#   bmcast_mac
# );

sub pcap_trace {
  my ($file, $bmcast_mac, $packet_callback, $callback_data, $callback_config, $posttrace_callback) = @_;
  my ($err, $pcap, %packet_config);
  $pcap = Net::Pcap::open_offline($file, \$err) or die "Cant read '$file': $err\n";
  %packet_config = (
    packet_callback => $packet_callback,
    callback_data => $callback_data,
    callback_config => $callback_config,
    bmcast_mac => $bmcast_mac,
    id => 1 
  );
  Net::Pcap::loop($pcap, -1, \&pcap_packet, \%packet_config);
  if ($posttrace_callback) {
    $posttrace_callback->($callback_data, $callback_config);
  }
  Net::Pcap::close($pcap);
}

sub pcap_live {
  # TODO Make it so the live data is written to a pcap file
  my ($device, $bmcast_mac, $packet_callback, $callback_data, $callback_config, $posttrace_callback) = @_;
  my ($err, $pcap, %packet_config);
  $pcap = Net::Pcap::open_live($device, 1024, 1, 0, \$err) or die "Cant read device '$device': $err\n";
  %packet_config = (
    packet_callback => $packet_callback,
    callback_data => $callback_data,
    callback_config => $callback_config,
    bmcast_mac => $bmcast_mac,
    id => 1
  );
  Net::Pcap::loop($pcap, -1, \&pcap_packet, \%packet_config);
  if ($posttrace_callback) {
    $posttrace_callback->($callback_data, $callback_config);
  }
  pcap_close($pcap);
}

sub pcap_packet {
  my ($pcap_config, $header, $binpacket) = @_;
  my (%packet, $header_length);
  $header_length = cap_header_length($binpacket);
  $packet{ID} = $pcap_config->{id}++;
  $packet{TIME} = $header->{'tv_sec'} + ($header->{'tv_usec'} / 1000000);
	$packet{TYPE} = (ord(substr($binpacket, $c80211bin{FRAME_CONTROL} + $header_length, 1)) 
    >> $c80211bin{TYPE_SHIFT}) & $c80211bin{TYPE_MASK};
  $packet{SUBTYPE} = (ord(substr($binpacket, $c80211bin{FRAME_CONTROL} + $header_length, 1)) 
    >> $c80211bin{SUBTYPE_SHIFT}) & $c80211bin{SUBTYPE_MASK};
  if ($packet{TYPE} == $c80211types{MAN} || $packet{TYPE} == $c80211types{DATA}) {
    my ($a1, $a2, $a3, $ds);
    $a1 = binaryMAC_to_string(substr($binpacket, $c80211bin{A1} + $header_length, 6));
    $a2 = binaryMAC_to_string(substr($binpacket, $c80211bin{A2} + $header_length, 6));
    $a3 = binaryMAC_to_string(substr($binpacket, $c80211bin{A3} + $header_length, 6));
    $ds = ((ord(substr($binpacket, $c80211bin{FRAME_CONTROL} + 1 + $header_length, 1))) & $c80211bin{DS_MASK});
    if ($packet{TYPE} == $c80211types{MAN}) {
      if ($packet{SUBTYPE} == $c80211subtypes{BEACON} && is_64_bit()) {
        $packet{BEACON_TIME} = unpack("Q", substr($binpacket, $c80211bin{TIMESTAMP} + $header_length, 8));
      }
      $packet{BSSID_MAC} = $a3;
      $packet{SRC_MAC} = $a2;
    }
    else {
      $packet{BSSID_MAC} = data_bssid($a1, $a2, $a3, $ds);
      $packet{SRC_MAC} = data_src($a1, $a2, $a3, $ds, $pcap_config->{bmcast_mac});
    }
    $packet{SEQNUM} = unpack("v", substr($binpacket, $c80211bin{SEQNUM} + $header_length, 2)) 
      >> $c80211bin{SEQNUM_SHIFT};
    $packet{RETRY} = ((ord(substr($binpacket, $c80211bin{FRAME_CONTROL} + 1 + $header_length, 1))) 
      & $c80211bin{RETRY_MASK}) >> $c80211bin{RETRY_SHIFT}; 

    #print "$packet{ID} $packet{SRC_MAC} $packet{BSSID_MAC}\n";
    $pcap_config->{packet_callback}->(\%packet, $pcap_config->{callback_data}, $pcap_config->{callback_config});
  }
}

sub binaryMAC_to_string {
	my ($binary_mac) = @_;
	return sprintf("%02x:%02x:%02x:%02x:%02x:%02x", 
									ord(substr($binary_mac, 0, 1)),
									ord(substr($binary_mac, 1, 1)),
									ord(substr($binary_mac, 2, 1)),
									ord(substr($binary_mac, 3, 1)),
									ord(substr($binary_mac, 4, 1)),
									ord(substr($binary_mac, 5, 1)));
}

sub cap_header_length {
  my ($binpacket) = @_;
  my ($avs, $pcap);

  $avs = unpack("N", substr($binpacket, $c80211bin{HEADER_ID}, 4));
  $pcap = unpack("N", substr($binpacket, $c80211bin{HEADER_ID}, 4));
  if ($avs == $c80211bin{AVS_HEADER_ID}) {
    return $c80211bin{AVS_HEADER_LENGTH};
  }
  if ($pcap == $c80211bin{PRISM_HEADER_ID}) {
    return $c80211bin{PRISM_HEADER_LENGTH};
  }
  return 0;
}

sub is_64_bit {
  return ($Config{use64bitint} eq 'define' || $Config{longsize} >= 8);
}

1;
__END__
=head1 NAME

Wifidelity::Pcap - Perl extension for 802.11 Pcap Traces

=head1 SYNOPSIS

  use Wifidelity::Pcap;

=head1 DESCRIPTION

Stub documentation for Wifidelity::Pcap, created by h2xs.

=head2 EXPORT

=head1 SEE ALSO

=head1 AUTHOR

Aaron Schulman, E<lt>schulman@cs.umd.edu<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Aaron Schulman

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.


=cut
