package Wifidelity::Stats;

use Wifidelity::80211;

use 5.008008;
use strict;
use warnings;

require Exporter;
our @ISA = qw(Exporter);

our @EXPORT_OK = ('update_stats','reset_stats','update_print_stats_on_beacon', 'print_stat_totals');

our @EXPORT = qw();

our $VERSION = '0.01';

# Statistics data structures
#
# %statistics = ( 
#   
#   STA_BACK,
#   STA_PREV_SEQNUM,
#   STA_11E_PREV_SEQNUM,
#   STA_LAST_SEEN,
#   
#   PACKETS,
#   PACKETS_SEQNUM_CHANGE,
#   PACKETS_MISSING,
#   PACKETS_SEQNUM_BAD,
#   RETRY,
#   TIME,
#   BEACON_INTERVAL
#
# ) 
#
# %config = ( 
#
#   PROB,   
#   BSSID_MAC,
#   STA_EXPIRE_S,
#   SEQNUM_ROLLOVER_DIFF,
#   SEQNUM_MAX,
#   FRAME_TIME_MAX_S
# ) 

# UGLY
my %prev_stats;
reset_stats(\%prev_stats);

sub update_print_stats_on_beacon {
  my ($packet, $stats, $config) = @_;
  update_stats($packet, $stats, $config);
  if ($packet->{BSSID_MAC} eq $config->{BSSID_MAC} && $packet->{TYPE} == $c80211types{MAN} && $packet->{SUBTYPE} == $c80211subtypes{BEACON}) { 
    my %cmp_stats;
    if ($prev_stats{BEACON_INTERVAL} > 0) {
      cmp_stats(\%prev_stats, $stats, \%cmp_stats);
      print_stats(\%cmp_stats, $packet->{TIME}, $packet->{ID});
    }
    clone_stats($stats, \%prev_stats);
  }
}

sub print_stat_totals {
  my ($stats, $config) = @_;
  print "Packets $stats->{PACKETS}\n";
  print "Missing Packets $stats->{PACKETS_MISSING}\n";
  print "Beacon Packets $stats->{BEACON_INTERVAL}\n";
  print "STA Timed out and Returned: $stats->{STA_BACK}\n";
  print "Bad SeqNum Packets: $stats->{PACKETS_SEQNUM_BAD}\n";
}

sub update_stats {
  my ($packet, $stats, $config) = @_;
  # 802.11 Man and Data packets have seqnums
  if ($packet->{TYPE} == $c80211types{MAN} || $packet->{TYPE} == $c80211types{DATA}) {

    # Only process originating from the specified BSSID
    if ($packet->{BSSID_MAC} eq $config->{BSSID_MAC}) {
      if ($packet->{TYPE} == $c80211types{MAN} && $packet->{SUBTYPE} == $c80211subtypes{BEACON}) {
        $stats->{BEACON_INTERVAL}++;
      }
      if (rand() <= $config->{PROB}) {
        $stats->{PACKETS}++;
        if ($packet->{RETRY}) {
          $stats->{RETRY}++;
        }
        # Have not seen this STA, or STA has expired
        if (!defined($stats->{STA_LAST_SEEN}->{$packet->{SRC_MAC}}) ||
          ($packet->{TIME} - $stats->{STA_LAST_SEEN}->{$packet->{SRC_MAC}} > $config->{STA_EXPIRE_S})) {
          $stats->{STA_BACK}++;
          update_sta_stats($stats, $packet, $config, 1);
        }
        else {
          update_sta_stats($stats, $packet, $config, 0);
        }
      }
    }
  }
}

sub update_sta_stats {
  my ($stats, $packet, $config, $force) = @_;
  my ($seqchange, $elapsed);
  my $qos = $packet->{TYPE} == $c80211types{DATA} && ($packet->{SUBTYPE} >= $c80211subtypes{QOS} || $packet->{SUBTYPE} >= $c80211subtypes{NULL});

  if (!$force) {
    if ($qos) {
      if (!defined($stats->{STA_11E_PREV_SEQNUM}->{$packet->{SRC_MAC}})) {
        $stats->{STA_11E_PREV_SEQNUM}->{$packet->{SRC_MAC}} = $packet->{SEQNUM};
      }
      $seqchange = $packet->{SEQNUM} - $stats->{STA_11E_PREV_SEQNUM}->{$packet->{SRC_MAC}};  
    }
    else {
      if (!defined($stats->{STA_PREV_SEQNUM}->{$packet->{SRC_MAC}})) {
        $stats->{STA_PREV_SEQNUM}->{$packet->{SRC_MAC}} = $packet->{SEQNUM};
      }
      $seqchange = $packet->{SEQNUM} - $stats->{STA_PREV_SEQNUM}->{$packet->{SRC_MAC}};  
      #print "S:$packet->{SEQNUM} PS:$stats->{STA_PREV_SEQNUM}->{$packet->{SRC_MAC}} C:$seqchange\n";
    }
    if ($seqchange < $config->{SEQNUM_ROLLOVER_DIFF}) {
      $seqchange = $config->{SEQNUM_MAX} + $seqchange;
	  }
    $elapsed = $packet->{TIME} - $stats->{STA_LAST_SEEN}->{$packet->{SRC_MAC}};
  }
  else {
    # Set forced values
    $seqchange = 1;
    $elapsed = $config->{FRAME_TIME_MAX_S};
  }
  if ($seqchange > 0 &&  $seqchange <= $elapsed/$config->{FRAME_TIME_MAX_S}) {
    my $test = $elapsed/$config->{FRAME_TIME_MAX_S};
    #print "FR:$test\n";
    $stats->{PACKETS_SEQNUM_CHANGE} += $seqchange;
    if ($qos) {
      $stats->{STA_11E_PREV_SEQNUM}->{$packet->{SRC_MAC}} = $packet->{SEQNUM}; 
    }
    else {
      $stats->{STA_PREV_SEQNUM}->{$packet->{SRC_MAC}} = $packet->{SEQNUM}; 
    }
    $stats->{PACKETS_MISSING} += $seqchange - (1 - $packet->{RETRY});
    $stats->{STA_LAST_SEEN}->{$packet->{SRC_MAC}} = $packet->{TIME};
  }
  else {
    # Remove this packet from the stats
    $stats->{PACKETS_SEQNUM_BAD}++;
    $stats->{PACKETS}--;
    $stats->{RETRY} -= $packet->{RETRY};
  }
}

sub cmp_stats {
  my ($prev_stats, $stats, $cmp_stats) = @_;
  $cmp_stats->{RETRY} = $stats->{RETRY} - $prev_stats->{RETRY};
  $cmp_stats->{PACKETS_SEQNUM_CHANGE} = $stats->{PACKETS_SEQNUM_CHANGE} - $prev_stats->{PACKETS_SEQNUM_CHANGE};
  $cmp_stats->{PACKETS} = $stats->{PACKETS} - $prev_stats->{PACKETS};
  $cmp_stats->{PACKETS_MISSING} = $stats->{PACKETS_MISSING} - $prev_stats->{PACKETS_MISSING};
  $cmp_stats->{PACKETS_SEQNUM_BAD} = $stats->{PACKETS_SEQNUM_BAD} - $prev_stats->{PACKETS_SEQNUM_BAD};
  $cmp_stats->{STA_BACK} = $stats->{STA_BACK} - $prev_stats->{STA_BACK};
  $cmp_stats->{BEACON_INTERVAL} = $stats->{BEACON_INTERVAL};
  $cmp_stats->{STA_LAST_SEEN} = $stats->{STA_LAST_SEEN};
}

sub clone_stats {
  my($from_stats,$to_stats) = @_;
  $to_stats->{PACKETS} = $from_stats->{PACKETS};
  $to_stats->{PACKETS_SEQNUM_CHANGE} = $from_stats->{PACKETS_SEQNUM_CHANGE};
  $to_stats->{PACKETS_MISSING} = $from_stats->{PACKETS_MISSING};
  $to_stats->{PACKETS_SEQNUM_BAD} = $from_stats->{PACKETS_SEQNUM_BAD};
  $to_stats->{RETRY} = $from_stats->{RETRY};
  $to_stats->{STA_BACK} = $from_stats->{STA_BACK};
  $to_stats->{STA_LAST_SEEN} = $from_stats->{STA_LAST_SEEN};
  $to_stats->{BEACON_INTERVAL} = $from_stats->{BEACON_INTERVAL};
}

sub print_stats {
  my ($stats, $time, $id) = @_;

  my $unique_sta = 0;
  foreach (keys(%{$stats->{STA_LAST_SEEN}})) {
    my $mac = $_;
    # Has this STA been seen since that last beacon interval
    if ($stats->{STA_LAST_SEEN}->{$mac} > $time - 0.1) {
      $unique_sta++;
    }
  }

  my $score = score($stats);
  print "$stats->{BEACON_INTERVAL} $stats->{PACKETS} $stats->{PACKETS_SEQNUM_CHANGE} $stats->{RETRY} $score $stats->{STA_BACK} $unique_sta $id\n";
}

sub score {
  my ($stats) = @_;
  if ($stats->{RETRY} + $stats->{PACKETS_SEQNUM_CHANGE} == 0) {
    return 0;
  }
  return $stats->{PACKETS} / ($stats->{PACKETS_SEQNUM_CHANGE} +  $stats->{RETRY});
}

sub reset_stats {
  my($stats) = @_;
  $stats->{BEACON_INTERVAL} = 0;
  $stats->{STA_BACK} = 0;
  $stats->{STA_PREV_SEQNUM} = {};
  $stats->{STA_11E_PREV_SEQNUM} = {};
  $stats->{STA_LAST_SEEN} = {};
   
  $stats->{PACKETS} = 0;
  $stats->{PACKETS_SEQNUM_CHANGE} = 0;
  $stats->{PACKETS_MISSING} = 0;
  $stats->{PACKETS_SEQNUM_BAD} = 0;
  $stats->{RETRY} = 0;
}

1;
__END__
=head1 NAME

Wifidelity::Stats - Perl extension for 802.11 Trace Stats 

=head1 SYNOPSIS

  use Wifidelity::Stats;

=head1 DESCRIPTION

Stub documentation for Wifidelity::Stats, created by h2xs. 

=head2 EXPORT

=head1 SEE ALSO

=head1 AUTHOR

Aaron Schulman, E<lt>schulman@cs.umd.edu<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Aaron Schulman

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.


=cut
