package Wifidelity::Timing;

use Wifidelity::80211;
use Time::HiRes qw(tv_interval);

use 5.008008;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

our @EXPORT_OK = ('update_clockdiff', 'update_print_clockdiff', 'reset_timing');

our @EXPORT = qw();

our $VERSION = '0.01';

# Timing data structures
#
# %timing = ( 
# 
#  BEACON_INTERVAL
#  BEACON_TIME_US
#  HOST_TIME_S
# ) 
#
# %config = ( 
#
#   BSSID_MAC,
# ) 

# UGLY
my %prev_timing;
reset_timing(\%prev_timing);

sub update_print_clockdiff {
  my ($packet, $timing, $config) = @_;
  if ($packet->{TYPE} == $c80211types{MAN} && $packet->{SUBTYPE} == $c80211subtypes{BEACON}) {
    update_clockdiff($packet, $timing, $config);
    if ($prev_timing{BEACON_INTERVAL} > 0) {
      my %cmp_timing;
      cmp_timing(\%prev_timing, $timing, \%cmp_timing);  
      print_timing(\%cmp_timing);
    }
    clone_timing($timing, \%prev_timing);
  }
} 

sub update_clockdiff {
  my ($packet, $timing, $config) = @_;
  if ($packet->{TYPE} == $c80211types{MAN} && $packet->{SUBTYPE} == $c80211subtypes{BEACON} 
  && $packet->{SRC_MAC} eq $config->{BSSID_MAC}) {
    $timing->{BEACON_TIME_US} = $packet->{BEACON_TIME};
    $timing->{HOST_TIME_S} = $packet->{TIME};
    $timing->{BEACON_INTERVAL}++;
  }
}

sub cmp_timing {
  my ($prev_timing, $curr_timing, $cmp_timing) = @_;
  $cmp_timing->{BEACON_TIME_US} = $curr_timing->{BEACON_TIME_US} - $prev_timing->{BEACON_TIME_US};
  $cmp_timing->{HOST_TIME_S} = $curr_timing->{HOST_TIME_S} - $prev_timing->{HOST_TIME_S};
  $cmp_timing->{BEACON_INTERVAL} = $curr_timing->{BEACON_INTERVAL};
}

sub clone_timing {
  my ($from_timing, $to_timing) = @_;
  $to_timing->{BEACON_INTERVAL} = $from_timing->{BEACON_INTERVAL};
  $to_timing->{BEACON_TIME_US} = $from_timing->{BEACON_TIME_US};
  $to_timing->{HOST_TIME_S} = $from_timing->{HOST_TIME_S};
}

sub print_timing {
  my ($timing) = @_;
  my ($host_time_us, $diff);
  $host_time_us = $timing->{HOST_TIME_S} * 1000000;
  $diff = $host_time_us - $timing->{BEACON_TIME_US};
  printf ("%d %d %d %d\n", $timing->{BEACON_INTERVAL}, $timing->{BEACON_TIME_US}, $host_time_us, $diff);
}

sub reset_timing {
  my ($timing) = @_;
  $timing->{BEACON_INTERVAL} = 0;
  $timing->{BEACON_TIME_US} = 0;
  $timing->{HOST_TIME_S} = 0;
}

1;
__END__
=head1 NAME

Wifidelity::Timing - Perl extension for 802.11 Trace Timing 

=head1 SYNOPSIS

  use Wifidelity::Timing;

=head1 DESCRIPTION

Stub documentation for Wifidelity::Timing, created by h2xs. 

=head2 EXPORT

None by default.

=head1 SEE ALSO

=head1 AUTHOR

Aaron Schulman, E<lt>schulman@cs.umd.edu<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007 by Aaron Schulman

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=cut
